package hero.entity;
import hero.interfaces.Constants;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnEdgeLocal;
import hero.util.BonitaProjectLocator;
import hero.util.BonitaServiceException;
import hero.util.BonitaServiceValue;
import hero.util.HeroException;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaProjectValue;

import java.util.*;

public class OrEdgeState extends EdgeState {

	private static OrEdgeState nd;

	public OrEdgeState() {
	}

	public static OrEdgeState getInstance() {
		if (nd == null) {
			nd = new OrEdgeState();
		}
		return nd;
	}

	public int state(BnNodeLocal node) {
		//Collection edges = node.getInBnEdges();
		BonitaProjectValue currentProject=null;
		Collection edges = new ArrayList();
		Collection edgesModel = new ArrayList();
		
		try{
			edges = BonitaServiceValue.getNode(node.getBnProject(),node.getName()).getInEdges();
		}catch (Exception e) {e.printStackTrace();}
		
		//	Instances consistency (may be some activities are not yet created in the instance)
		String nodeName = node.getName();
		if (isInstance(node.getName()))
			nodeName = getModel(nodeName);								
		try{
			if (node.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
				edgesModel = BonitaServiceValue.getNode(node.getBnProject().getModel().getName(),nodeName).getInEdges();
			else
				edgesModel = BonitaServiceValue.getNodeFromCache(node.getBnProject().getModel(),nodeName).getInEdges();
		}catch (Exception e) {edgesModel=new ArrayList();}
		if (edges.size() != edgesModel.size() && !edgesModel.isEmpty())
			edges = concatEdges(edges, edgesModel);		
	
		Iterator i = edges.iterator();
		int max = 0;
		int result = Constants.Nd.EDGEINITIAL;
		while (i.hasNext())
		{
			BonitaEdgeValue edge = (BonitaEdgeValue) i.next();

			if(edge.getState() == Constants.Ed.DEAD && max <= 0)
			{
				max = 0;
				result = Constants.Nd.CANCEL;
			}
			else if(edge.getState() == Constants.Ed.INITIAL && max <= 1)
			{
				max = 1;
				result = Constants.Nd.EDGEINITIAL;
			}
			else if(edge.getState() == Constants.Ed.ANTICIPATING && max <= 2)
			{
				max = 2;
				result = Constants.Nd.ANTACTIVE;
			}
			else if(edge.getState() == Constants.Ed.ACTIVE && max <= 3)
			{
				max = 3;
				result = Constants.Nd.ACTIVE;

				// As the highest possible value is reached 
				break;
			}
		}

		return result;
	}
	
	private String getModel(String instanceName) {
		int i = instanceName.indexOf("_instance");
		return (instanceName.substring(0, i));
	}

	private boolean isInstance(String name) {
		return (name.matches(".*_instance.*"));
	}
	
	private Collection concatEdges(Collection edges, Collection edgesModel) {
		Collection result = new ArrayList();
		Iterator ed = edgesModel.iterator();
		while (ed.hasNext())
		{
			boolean found = false;
			BonitaEdgeValue el = (BonitaEdgeValue)ed.next();
			String nodeName = el.getInNode();
			Iterator instEdges = edges.iterator();
			while(instEdges.hasNext())
			{
				BonitaEdgeValue insEd = (BonitaEdgeValue)instEdges.next();
				String node = insEd.getInNode();
				if (this.isInstance(node))
					node = this.getModel(node);
				if (nodeName.equals(node))
				{
					result.add(insEd);
					found = true;
					break;
				}
			}
			if (!found)
				result.add(el);
		}
		return result;
	}

}
